@@ -50,7 +50,7 @@ static unsigned long oil_cpu_flags;
 
 #if defined(__i386__)
 static char *
-get_cpuinfo (void)
+get_proc_cpuinfo (void)
 {
   char *cpuinfo;
   int fd;
@@ -71,16 +71,12 @@ get_cpuinfo (void)
 
 #ifdef __i386__
 static void
-oil_cpu_i386_getflags(void)
+oil_cpu_i386_getflags_cpuinfo (char *cpuinfo)
 {
-  char *cpuinfo;
   char *cpuinfo_flags;
   char **flags;
   char **f;
 
-  cpuinfo = get_cpuinfo();
-  if (cpuinfo == NULL) return;
-
   cpuinfo_flags = get_cpuinfo_flags_string(cpuinfo);
   if (cpuinfo_flags == NULL) {
     free (cpuinfo);
@@ -124,6 +120,100 @@ oil_cpu_i386_getflags(void)
   free (cpuinfo);
   free (cpuinfo_flags);
 }
+
+static void
+get_cpuid (uint32_t op, uint32_t *a, uint32_t *b, uint32_t *c, uint32_t *d)
+{
+  __asm__ (
+      "  pushl %%ebx\n"
+      "  cpuid\n"
+      "  mov %%ebx, %%esi\n"
+      "  popl %%ebx\n"
+      : "=a" (*a), "=S" (*b), "=c" (*c), "=d" (*d)
+      : "0" (op));
+}
+
+static void
+test_cpuid (void *ignored)
+{
+  uint32_t eax, ebx, ecx, edx;
+
+  get_cpuid (0x00000000, &eax, &ebx, &ecx, &edx);
+}
+
+static void
+oil_cpu_i386_getflags_cpuid (void)
+{
+  uint32_t eax, ebx, ecx, edx;
+  uint32_t level;
+  char vendor[13] = { 0 };
+  int ret;
+
+  oil_cpu_fault_check_enable ();
+  ret = oil_cpu_fault_check_try(test_cpuid, NULL);
+  oil_cpu_fault_check_disable ();
+  if (!ret) {
+    /* CPU thinks cpuid is an illegal instruction. */
+    return;
+  }
+
+  get_cpuid (0x00000000, &level, (uint32_t *)(vendor+0),
+      (uint32_t *)(vendor+8), (uint32_t *)(vendor+4));
+
+  OIL_DEBUG("cpuid %d %s", level, vendor);
+
+  if (level < 1) {
+    return;
+  }
+
+  get_cpuid (0x00000001, &eax, &ebx, &ecx, &edx);
+
+  /* Intel flags */
+  if (edx & (1<<15)) {
+    oil_cpu_flags |= OIL_IMPL_FLAG_CMOV;
+  }
+  if (edx & (1<<23)) {
+    oil_cpu_flags |= OIL_IMPL_FLAG_MMX;
+  }
+  if (edx & (1<<25)) {
+    oil_cpu_flags |= OIL_IMPL_FLAG_SSE;
+  }
+  if (edx & (1<<26)) {
+    oil_cpu_flags |= OIL_IMPL_FLAG_SSE2;
+  }
+  if (ecx & (1<<0)) {
+    oil_cpu_flags |= OIL_IMPL_FLAG_SSE3;
+  }
+  
+  if (memcmp (vendor, "AuthenticAMD", 12) == 0) {
+    get_cpuid (0x80000001, &eax, &ebx, &ecx, &edx);
+
+    /* AMD flags */
+    if (edx & (1<<22)) {
+      oil_cpu_flags |= OIL_IMPL_FLAG_MMXEXT;
+    }
+    if (edx & (1<<31)) {
+      oil_cpu_flags |= OIL_IMPL_FLAG_3DNOW;
+    }
+    if (edx & (1<<30)) {
+      oil_cpu_flags |= OIL_IMPL_FLAG_3DNOWEXT;
+    }
+  }
+}
+
+static void
+oil_cpu_i386_getflags(void)
+{
+  char *cpuinfo;
+
+  cpuinfo = get_proc_cpuinfo();
+  if (cpuinfo) {
+    oil_cpu_i386_getflags_cpuinfo(cpuinfo);
+  } else {
+    oil_cpu_i386_getflags_cpuid();
+  }
+}
+
 #endif
 
 #ifdef __powerpc__
@@ -243,9 +333,14 @@ _strndup (const char *s, int n)
 #endif
 
 static jmp_buf jump_env;
+#ifdef HAVE_SIGACTION
 static struct sigaction action;
 static struct sigaction oldaction;
+#else
+static void * oldhandler;
+#endif
 static int in_try_block;
+static int enable_level;
 
 static void
 illegal_instruction_handler (int num)
@@ -260,10 +355,17 @@ illegal_instruction_handler (int num)
 void
 oil_cpu_fault_check_enable (void)
 {
-  memset (&action, 0, sizeof(action));
-  action.sa_handler = &illegal_instruction_handler;
-  sigaction (SIGILL, &action, &oldaction);
-  in_try_block = 0;
+  if (enable_level == 0) {
+#ifdef HAVE_SIGACTION
+    memset (&action, 0, sizeof(action));
+    action.sa_handler = &illegal_instruction_handler;
+    sigaction (SIGILL, &action, &oldaction);
+#else
+    signal (SIGILL, illegal_instruction_handler);
+#endif
+    in_try_block = 0;
+  }
+  enable_level++;
 }
 
 int
@@ -284,6 +386,13 @@ oil_cpu_fault_check_try (void (*func) (void *), void *priv)
 void
 oil_cpu_fault_check_disable (void)
 {
-  sigaction (SIGILL, &oldaction, NULL);
+  enable_level--;
+  if (enable_level == 0) {
+#ifdef HAVE_SIGACTION
+    sigaction (SIGILL, &oldaction, NULL);
+#else
+    signal (SIGILL, oldhandler);
+#endif
+  }
 }
 
