@@ -89,11 +89,13 @@ oil_optimize_all (void)
   OilFunctionClass *klass;
   int i;
 
+  oil_cpu_fault_check_enable ();
   for (i = 0; i < _oil_n_function_classes; i++) {
     klass = oil_class_get_by_index (i);
 
     oil_class_optimize (klass);
   }
+  oil_cpu_fault_check_disable ();
 }
 
 /**
@@ -245,6 +247,10 @@ oil_class_optimize (OilFunctionClass * klass)
 
   OIL_DEBUG ("optimizing class %s", klass->name);
 
+  if (klass->reference_impl == NULL) {
+    OIL_ERROR ("class %s has no reference implmentation", klass->name);
+    return;
+  }
   if (klass->first_impl == NULL) {
     OIL_ERROR ("class %s has no implmentations", klass->name);
     return;
@@ -278,6 +284,11 @@ oil_class_optimize (OilFunctionClass * klass)
       impl->flags |= OIL_IMPL_FLAG_DISABLED;
     }
   }
+  if (min_impl == NULL) {
+    OIL_ERROR ("failed to find optimal implementation for class %s",
+        klass->name);
+    return;
+  }
   klass->chosen_impl = min_impl;
   klass->func = min_impl->func;
 
@@ -340,6 +351,20 @@ oil_class_register_impl_by_name (const char *klass_name, OilFunctionImpl *impl)
   klass = oil_class_get (klass_name);
   if (klass == NULL) return;
 
+  oil_class_register_impl (klass, impl);
+}
+
+/**
+ * oil_class_register_impl:
+ * @klass: the class
+ * @impl: an implementation
+ *
+ * Adds @impl to the list of implementations associated with
+ * the function class given by @klass.
+ */
+void
+oil_class_register_impl (OilFunctionClass *klass, OilFunctionImpl *impl)
+{
   impl->klass = klass;
   impl->next = impl->klass->first_impl;
   klass->first_impl = impl;
@@ -350,3 +375,29 @@ oil_class_register_impl_by_name (const char *klass_name, OilFunctionImpl *impl)
   }
 }
 
+/**
+ * oil_class_register_impl_full:
+ * @klass: the class
+ * @func: the function
+ * @name: name of the function
+ * @flags: CPU flags
+ *
+ * Adds @func to the list of implementations associated with
+ * the function class given by @klass.
+ */
+void
+oil_class_register_impl_full (OilFunctionClass *klass,
+    void (*func)(void), const char *name, unsigned int flags)
+{
+  OilFunctionImpl *impl;
+
+  impl = malloc(sizeof(OilFunctionImpl));
+  memset (impl, 0, sizeof(OilFunctionImpl));
+
+  impl->func = func;
+  impl->flags = flags;
+  impl->name = strdup(name);
+
+  oil_class_register_impl(klass,impl);
+}
+
